//------------------------------------------------------------------------------
//
//  npdata.h
//  npe
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------

#ifndef npdata_H_
#define npdata_H_

// #define _LINUX_
#define _MSW_
// #define _OSX_


#include <freeglut.h>

#include "data/stdbool.h"


#define kKeyMapSize 256
#define kKeyEventTypeSize 4

#define	kPlotMax 4096
#define kGridCount 10
#define kDefaultScale = 1.0




enum {
	kAverageNull = 0,
	kAverage,
	kAverageRootMeanSquare,
	kAverageSmoothed,
	kAverageRunning
};

enum {
	kLeftJustify = 0,
	kRightJustify,
	kCenterJustify
};

enum {
	kSmallFont = 0,
	kMediumFont,
	kLargeFont
};

enum
{
	kChannelNull = 0,
	kChannelCycleCount,
	kChannelOscilloscope
};

struct Key {
	int		modShift;
	int		modCtrl;
	int		modAlt;
	int		modCommand;
	int		map[kKeyEventTypeSize][kKeyMapSize];
};
typedef struct Key Key;
typedef struct Key * pKey;

struct DataMap {
	int		size;		// memory used

	void*	plot[kPlotMax];
	int		plotCount;
	int		plotIndex;
};
typedef struct DataMap DataMap;
typedef struct DataMap * pDataMap;

struct IO {
	int		size;
	Key		key;
};
typedef struct IO IO;
typedef struct IO * pIO;

struct Ctrl {
	int		size;
};
typedef struct Ctrl Ctrl;
typedef struct Ctrl * pCtrl;

struct Data {
	DataMap map;
	IO		io;
	Ctrl	ctrl;
};
typedef struct Data Data;
typedef struct Data * pData;


typedef struct {
   GLdouble x,y,z;
} recVec;

typedef struct {
	GLfloat x,y,z;
} GLfloatXYZ;

typedef struct {
	GLfloat angle,x,y,z;
} GLfloatAXYZ;

typedef struct {
	GLfloat r,g,b,a;
} GLfloatRGBA;

typedef struct {
	int x,y,z;
} IntXYZ;

typedef struct {
	GLubyte r,g,b,a;
} GLubyteRGBA;

typedef struct {
	GLboolean x,y,z;
} BoolXYZ;

typedef struct {
	GLboolean angle,x,y,z;
} BoolAXYZ;


//-----------------------------------------------------------------------------
enum
{	
	kPlotDefault,			//simplest type, no plot data
	kPlotPoints,			//for X/XY/XYZ line, eeg/ecg, temp, movement...
	kPlotPin,				//combo of cones, spheres, toroids, etc...
	kPlotCamera,			//for both real-world and virtual GL cameras 
	kPlotVideo,				//video from live/stream, internal GL, or file
	kPlotGrid				//grid in 1D, 2D or 3D
};

enum 
{
	kSurfaceNull = 0,
	kSurfacePlane,
	kSurfaceCylinder,
	kSurfaceCone,
	kSurfaceSphere,
	kSurfaceTorus,
	kSurfaceMesh,							// terrain
	kSurfaceLayers,							// ie stack of FFT images, MRI...
	kSurfaceCamera
};

enum 
{
	kShadingNull = 0,
	kShadingWire,
	kShadingFlat,
	kShadingGouraud,
	kShadingPhong,
	kShadingReflection,
	kShadingRaytrace
};

enum
{
	kCameraSensorNull = 0,					//don't know the sensor type
	kCameraSensorMonoCCD,					//monochrome CCD such as BW, IR
	kCameraSensorDebayerCCD,				//single color CCD debayer pattern
	kCameraSensorThreeCCD,					//3CCD color sensor
	kCameraSensorDepthCCD					//BW or color with Z depth data
};

enum
{
	kVideoFormatNull = 0,
	kVideoFormatNTSC
};

enum
{
	kColorSpaceNull = 0,
	kColorSpaceRGB_8
};


// plots are organized as a binary tree, supports complexity of linked objects
// ie a recPlot of type plotCamera may have a multiple depth daughter tree that
// supports severa cameras or videoPlots for circle cam rigs etc...
//-----------------------------------------------------------------------------
struct NPplot
{
	int			type;						//plot type, point, pin, cam, video
	void*		data;					//plot type specific data

	void*		parent;						//parent plot, binary tree of plots
	void*		branchL;					//the left daughter branch
	void*		branchR;					//the right daughter branch

	IntXYZ		channel;					//channel to use for x,y,z points
	IntXYZ		channelIndex;				//last updated channel index

	IntXYZ		averageType;				//averaging type applied to data
	IntXYZ		sampleInterval;				//the interval to be averaged

	GLfloatAXYZ	rotate;						//orientation
	GLfloatXYZ	scale;						//scale plot XYZ
	GLfloatXYZ	translate;					//location relative to plotOrigin
	GLfloatXYZ	plotOrigin;					//origin relative to parent origin

	GLfloatAXYZ	rotateRate;					//rotation rate
	GLfloatXYZ	scaleRate;					//scale rate 
	GLfloatXYZ	translateRate;				//pans the location of the plot, pronunciar en Espanol, lol

	int			surfaceType;				//FFT layers, terrain, sphere...
	int			shaderType;					//wire, flat, gouraud, phong

	GLfloat		lineWidth;					//draws lines, zero for off
	GLfloat		pointSize;					//draws dots, zero for off
	GLfloat		circle;						//draw circle at current data point

	int			colorIndex;
	GLubyteRGBA	color;						//color assigned to new data
	GLboolean	colorFade;

	GLboolean	hide;						//hide the plot, data stays active
	GLboolean	freeze;						//freezes data and physics updates
	GLboolean	center;						//centers to plotOrigin
	GLboolean	centerPerGrid;				//centers in gridSpace increments

	BoolXYZ		autoZoom;					//scales plot to fit screen
	GLboolean	scroll;						//scroll right to left
	
	BoolXYZ		triggerOnHi;				//triggers are multipurpose
	BoolXYZ		triggerOnLow;
	GLfloatXYZ	triggerLevelHi;
	GLfloatXYZ	triggerLevelLow;
	int			triggerCommand;				//execute ctrlCommand upon trigger
	
	BoolXYZ		proximity;					//used for collision engine 
	GLfloatXYZ	proximityRadius;

	BoolXYZ		drawCurrentValue;			//draw current values as text
	BoolXYZ		drawTranslate;				//draw translate as text
	BoolXYZ		drawScale;					//draw scale as text
	BoolAXYZ	drawRotate;					//draw rotation as text
};
typedef struct NPplot NPplot;
typedef struct NPplot *NPplotPtr;


//-----------------------------------------------------------------------------
struct NPpoints
{
	GLfloat		*vertexArray;				//the array of data points
	GLubyte		*colorArray;				//each point gets color assigned
	int			vertexCount;				//number of points in the plot
	int			arrayIndex;					//current vertex index in the array
};
typedef struct NPpoints NPpoints;
typedef struct NPpoints *NPpointsPtr;


//-----------------------------------------------------------------------------
struct NPpin
{
	void* textureRef;		//texture assigned to element
};
typedef struct NPpin NPpin;
typedef struct NPpin *NPpinPtr;


// cameras can be GL scene cameras or real-world cameras
// video is attached as a daughter of the recPlot
//-----------------------------------------------------------------------------
struct NPcamera
{
	void*		textureID;					//texture ID in GL

	GLfloatXYZ	target;						//object tracking in Global coord
	GLfloat		zoomFOV;					//35mm, 70mm...

	int			format;						//320p, 480i, 720p, 4K full app
	int			width;						//in pixels
	int			height;
	GLfloat		aspectRatio;				//1.0, 1.333, 1.777, 1.85, 2.25... 
	int			colorSpace;					//8, 12bit, YUV, RGBA, XYZ, CMYK...
	GLfloat		fps;						// 15, 24, 29.97, 30, 59.94, 120...

	GLfloat		aperture;					//F stop
	GLfloat		exposure;					//in seconds
	int			sensorType;					//3CCD, Debayer pattern...
};
typedef struct NPcamera NPcamera;
typedef struct NPcamera *NPcameraPtr;


//-----------------------------------------------------------------------------
struct NPvideo
{
	void*		textureID;					//texture ID in GL

	int			format;						//320p, 480i, 720p, 4K full app
	int			width;						//in pixels
	int			height;
	GLfloat		aspectRatio;				//1.0, 1.333, 1.777, 1.85, 2.25... 
	int			colorSpace;					//8, 12bit, YUV, RGBA, XYZ, CMYK...
	GLfloat		fps;						// 15, 24, 29.97, 30, 59.94, 120...

	GLfloatXYZ	bias;						//color shift
	GLfloatXYZ	gain;						//color multiplier
};
typedef struct NPvideo NPvideo;
typedef struct NPvideo *NPvideoPtr;


//-----------------------------------------------------------------------------
struct NPgrid
{
	IntXYZ		segments;
	GLfloatXYZ	size;
	GLboolean	overlay;					//overlays relative to screen coord
};
typedef struct NPgrid NPgrid;
typedef struct NPgrid * NPgridPtr;


//--- Functions ----------------------------------------------------------------

void npInitData (int argc, char* argv[], void* data);		// inits default
void npCloseData();

void* npDataGetDataRef();

void npDataUpdate (void* dataRef);

#endif

